#ifndef D3N2_STATICOBSTACLESIN3D_H
#define D3N2_STATICOBSTACLESIN3D_H

// Standard includes
#include <stdio.h>
#include <math.h>
#include <vector>
#include <cstdlib>
#include <armadillo>
#include <fstream>
#include "yaml-cpp/yaml.h"
#include "muParser.h"
#include <gsl/gsl_errno.h>
#include <gsl/gsl_interp.h>
#include <GL/glut.h>

// Local includes
#include "homotopy_tools.h"
#include "topology_tools.h"
#include "subh_utils.h"
#include "homotopy_NN_Astar.h"

// Definations
#define SET_OBSTACLE_COLOR_GL  glColor3f(0.85, 1.0, 0.5)

// Declarations
class D3N2_StaticObstacleIn3D_Env;
// --------------------------------

class muExpression
{
public:
	std::string s;
	mu::Parser* p;
	double* t;
	muExpression() { };
	muExpression(std::string expr);
	void init(std::string expr);
	double eval(double tt=0.0);
	// TODO: Add copy constructor, = operator and destructor. Without those there is a memory leak every time 'init' is called.
};

class muCoordinate
{
public:
	std::vector<muExpression> coords;
	arma::vec eval(double tt=0.0);
};

void operator >> (const YAML::Node& node, muExpression& ex);
void operator >> (const YAML::Node& node, muCoordinate& co);
void operator >> (const YAML::Node& node, arma::vec& v);

// ================================================

class GenericObstacle
{
public:
	std::string shape;
	std::string homotopy;
	D3N2_StaticObstacleIn3D_Env* parentEnv;
	// --------------------------------------------
	std::vector< muCoordinate > skeleton;
	virtual bool isPointInside(arma::vec coord, double tt=0.0) = 0;
	virtual void draw_gl(double tt=0.0) = 0;
	// --------------------------------------------
	double padding;
};

// ---------------------------------
// LOOP:

class Obstacle_loop: public GenericObstacle
{
public:
	muExpression rad;
	Obstacle_loop(const YAML::Node& envFileNode);
	bool isPointInside(arma::vec coord, double tt=0.0);
	void draw_gl(double tt=0.0);
};

// ---------------------------------
// BOX:

class Obstacle_box: public GenericObstacle
{
public:
	muCoordinate v1, v2;
	Obstacle_box(const YAML::Node& envFileNode);
	bool isPointInside(arma::vec coord, double tt=0.0);
	void draw_gl(double tt=0.0);
};

// ================================================

class Traj_T
{
public:
	int n; // Number of points
	int d; // Dimension of points
	std::vector< double* > coords;
	double* t;
	arma::vec homotopySignature;
	
	bool interp_initiated;
	std::vector< gsl_interp* > workspaces;
	std::vector< double > getCoords(double tt);
	
	bool allocated;
	Traj_T() { interp_initiated = false; allocated = false; };
	void _alloc(int nn, int dd);
	void _free(void);  // Needs to be called manually!!
};

void operator >> (const YAML::Node& node, Traj_T& traj);

// ================================================

class D3N2_StaticObstacleIn3D_Env
{
public:
	// It is important that we maintain only one instance of D3N2_StaticObstacleIn3D_Env at a time
	
	D3N2_StaticObstacleIn3D_Env();
	void parseEnvFile(const char* fName, bool vis=false);
	void parseObstacle(const YAML::Node& nObs);
	
	std::vector<GenericObstacle*> obstacles;
	std::vector<Traj_T> trajectories;
	
	// problem block
	std::string problem_description;
	// planoptions block
	DiscContTransform planTransform;
	arma::vec startcoord, stopcoord;
	int exploreMaxCount;
	std::vector<arma::vec> allowed_signatures, blocked_signatures;
	bool onlyJordan, dontExploreNonjordanStates;
	std::vector<double> mins_planner, maxs_planner, cellsize_planner;
	double SingularityManifoldJiggle;
	bool precomputeAndReplan;
	
	// drawoptions block
	int winsizeX, winsizeY;
	arma::vec camerapos, lookat;
	double fovY;
};

// ---------------------------
// Utility functions

void glDrawCylinderBetweenPoints ( arma::vec p1, arma::vec p2, double radius );
void glDrawBox ( arma::vec p1, arma::vec p2 );

#endif

