#ifndef SUBH_UTILS_H
#define SUBH_UTILS_H

#include <stdio.h>
#include <math.h>
#include <vector>
#include <armadillo>
#include <stdlib.h>
#include <stdarg.h>
#include <ctime>
#include <opencv/cv.h>
#include <opencv/cvaux.h>
#include "yaml-cpp/yaml.h"

// Macros
#define __max(x,y) ((x)>(y)?(x):(y))
#define __min(x,y) ((x)>(y)?(y):(x))
#define _round(r) (((r)>0.0)?((int)floor((r)+0.5)):((int)ceil((r)-0.5)))
#define _sign(x) ( ((double)(x)>0.0) ? 1 : ( ((double)(x)<0.0) ? -1 : 0 ) )
#define _RAND_11  ( ((double)rand()) / ((double)RAND_MAX) )

#define PI_CONST 3.141592654
#define SQRT2_CONST 1.414213562
#define SQRT3_CONST 1.732050808
#define SQRT4_CONST 2.0

// ============================================================
// Helper functions

// For planning, plotting, discretization, etc

class DiscContTransform
{
public:
	int dim;
	std::vector<double> cont_min, cont_max;
	std::vector<int> disc_size;
	std::vector<double> cellsize;
	
	DiscContTransform() { };
	DiscContTransform(std::vector<double> mins, std::vector<double> maxs, double cs);
	DiscContTransform(std::vector<double> mins, std::vector<double> maxs, std::vector<double> cs);
	void init(std::vector<double> mins, std::vector<double> maxs, double cs);
	void init(std::vector<double> mins, std::vector<double> maxs, std::vector<double> cs);
	
	std::vector<int> cont2disc(std::vector<double> cont);
	std::vector<int> cont2disc_arma(arma::vec cont);
	std::vector<double> disc2cont(std::vector<int> disc);
	arma::vec disc2cont_arma(std::vector<int> disc);
	
	//int scale_cont2disc(double cont) { return ( _round(cont/cellsize)); };
	//double scale_disc2cont(int disc) { return ((float)disc*cellsize); };
};

// Inter-library conversions

std::vector<double> _vStd_d(int count, ... );
std::vector<int> _vStd_i(int count, ... );
arma::vec _vArma(int count, ... );
arma::vec Std2Arma_vec(std::vector<double> vin);
std::vector<double> Arma2Std_vec(arma::vec vin);

// STD vector creator
template <class T>
std::vector<T> _vStd(int count, ... )
{
	std::vector<T> ret(count);
	va_list listptr;
	va_start(listptr, count); 
	for(int i=0; i<count; ++i)
		ret[i] = va_arg(listptr, T);
	return ret;
}

// Utils for arma
arma::vec armaCross(arma::vec v1, arma::vec v2);
arma::mat armaVertCast(arma::mat top, arma::mat bottom);

/*
template <class T>  std::vector<T> Array2StdVec(T* arr, int size);
template <class T>  void StdVec2AllocatedArray(std::vector<T> sv, T* arr); */

template <class T>
std::vector<T> Array2StdVec(T* arr, int size)
{
	std::vector<T> ret(size);
	for (int a=0; a<size; a++)
		ret[a] = arr[a];
	return ret;
}

template <class T>
void StdVec2AllocatedArray(std::vector<T> sv, T* arr)
{
	for (int a=0; a<sv.size(); a++)
		arr[a] = sv[a];
}

// For homotopy class problem

std::vector<int> set_k(int D, int k); // Returns a set {0, 1, ..., D-1}, without k in it. Thus, 0 <= k < D
double computePermSign(std::vector<int> p); // the vector p should contain a permutation of { 0, 1, 2, ..., p.size()-1 }.
double analytic_Level1_Integration(double L, double U, double p, double q, double a, double b, double c, int D);

// String manipulation function
std::string CreateTimeStampedDerivedFilename(std::string originalFname, std::string extension);

// For GSL

class gslErrorInfo
{
public:
	bool wasErrorOnLastIntegration;
	const char* reason;
	const char* file;
	int line;
	int gsl_errno;
};

// Geometry operations
std::vector<arma::vec> RotateAndTranslate( std::vector<arma::vec> inPoints, 
											arma::vec RotAxisBase, arma::vec RotAxisApex, double RotAngle, 
											arma::vec Translate );
											
// for geometric computations in 2D
arma::vec FindIntersectionBetweenSegments( std::vector< arma::vec > p1, std::vector< arma::vec > p2 );

// ====================================
// YAML-cpp

template <class T>
void operator >> (const YAML::Node& node, T& varref)
{
	/*std::string expr;
	node >> expr;
	ex.init(expr);*/
	varref = node.as<T>();
}

#define FindValue(key)  operator[](key)

#endif

