#ifndef TOPOLOGY_TOOLS_H
#define TOPOLOGY_TOOLS_H

// Includes
#include <stdio.h>
#include <math.h>
#include <vector>
#include <armadillo>

#include "subh_utils.h"

/* Class declarations */

class BasicSimplex
{
public:

	// Basic variables describing the simplex
	int dim_ambient, dim_simplex;
	std::vector<arma::vec> v;  // (dim_simplex+1) vectors, each of size dim_ambient. access: this->v[vertexNo][coordNo] (for read only)
	int orientation; // 1 or -1. By default it is 1. Use setSameOrientation to set orientation same as an adjascent simplex.
	
	// Constructors and initiations
	BasicSimplex() {};
	BasicSimplex(int D_simplex, int D_ambient); // Initiation 0. Constructor
	void setVertex(int vNo, arma::vec thisV) { v[vNo] = thisV; foundTransformation = false; foundVolumescales = false; };  // Initiation 1.
																  //vNo = 0,1,2,...,dim_simplex  (dim_simplex+1 vertices)
	int setSameOrientation(BasicSimplex* anotherSimplex, double VertexSepThresh=1e-6); // Initiation 2a. anotherSimplex must have dim_simplex vertices common to *this.
	
	// Transformations for finding normalized (local) coordinates:
	//		v[k] = NormalizationMatrix * v_normalized[k] + NormalizationVector
	//		v[0] is mapped to [0,0,...,0,0]; v[1] is mapped to [1,0,...,0]; v[2] is mapped to [1,1,...,0]; etc.
	arma::vec NormalizationVector;
	arma::mat NormalizationMatrix;
	// Volume scales
	std::vector< std::vector<double> > volScales; // Use: volScales[k][rho]
	
	// functions for finding transformations to normalized coords and volume scales
	bool foundTransformation, foundVolumescales;
	void preprocess(bool findVolumeScales=true, std::vector< std::vector<int> >* volumeScalePartitions=NULL);
																// Initiation 2b. To be called when all the vertices are set
	
	// Other functions
	void print(std::string header="");
	
	// Note: Order of initiation - 0, 1. Then 2a and 2b are independednt and their order can be exchanged.
};

// -------------------------------------------------------------

class SimplicesList // Can be used to describe a basic manifold without connection or structural information
{
public:
	std::vector<BasicSimplex> s;
	// TODO: Add '+' operator, etc.
	void printSimplices_compact(void);
};

// Function for creating tori embeded in a d-dimensional ambient space
// WARNING: Simplices returned are NOT preprocessed!!
SimplicesList Create_1torus(std::vector< arma::vec > vertices);
SimplicesList Create_OpenCurve(std::vector< arma::vec > vertices);
SimplicesList Create_2torus(std::vector< std::vector< arma::vec > > vertices);


#endif

